import HttpRequestMock from 'http-request-mock';
import fs from 'fs';
const mocker = HttpRequestMock.setup();
const fetchMocker = HttpRequestMock.setupForFetch();
const cwd = process.cwd();
let mockQueue = {};
let printMsg = false;
let knownErr = false;
// ░█████╗░░█████╗░███╗░░██╗████████╗██████╗░░█████╗░██╗░░░░░
// ██╔══██╗██╔══██╗████╗░██║╚══██╔══╝██╔══██╗██╔══██╗██║░░░░░
// ██║░░╚═╝██║░░██║██╔██╗██║░░░██║░░░██████╔╝██║░░██║██║░░░░░
// ██║░░██╗██║░░██║██║╚████║░░░██║░░░██╔══██╗██║░░██║██║░░░░░
// ╚█████╔╝╚█████╔╝██║░╚███║░░░██║░░░██║░░██║╚█████╔╝███████╗
// ░╚════╝░░╚════╝░╚═╝░░╚══╝░░░╚═╝░░░╚═╝░░╚═╝░╚════╝░╚══════╝
export function start() {
    fetchMocker.mock({
        url: /./,
        response(opts) {
            let results = mockAPI(opts);
            if (!results) {
                //@ts-ignore
                return this.bypass();
            }
            this.status = results.status;
            if (results.headers) {
                this.headers = results.headers;
            }
            return results.body;
        }
    });
    mocker.mock({
        url: /./,
        response(opts) {
            let results = mockAPI(opts);
            if (!results) {
                //@ts-ignore
                return this.bypass();
            }
            this.status = results.status;
            if (results.headers) {
                this.headers = results.headers;
            }
            return results.body;
        }
    });
}
export function stop() {
    mocker.reset();
    fetchMocker.reset();
    mockQueue = {};
}
export function getCounts(service) {
    if (!mockQueue.hasOwnProperty(service)) {
        itWereBad(`No service ${service}`);
    }
    return mockQueue[service].counts;
}
export function getQueuedItems() {
    let count = 0;
    for (const [service, serviceDeets] of Object.entries(mockQueue)) {
        count += (serviceDeets.get.length + serviceDeets.put.length + serviceDeets.post.length + serviceDeets.patch.length);
    }
    return count;
}
export function printQueue() {
    if (Object.keys(mockQueue).length === 0) {
        return "No Services";
    }
    else {
        let printArr = [];
        for (const [service, serviceDeets] of Object.entries(mockQueue)) {
            let messages = [];
            for (const [reqType, requests] of Object.entries(serviceDeets)) {
                if (["records", "cases", "counts", "rateLimiting"].includes(reqType)) {
                    continue;
                }
                for (const request of requests) {
                    messages.push(`  ${reqType} -> ${request.caseName}`);
                }
            }
            if (messages.length > 0) {
                printArr.push(`Service: ${service}`);
                for (const msg of messages) {
                    printArr.push(msg);
                }
            }
        }
        if (printArr.length === 0) {
            return "No outstanding Q items";
        }
        else {
            return printArr.join("\n");
        }
    }
}
export function printToConsole(setting) {
    if (typeof setting !== "boolean") {
        itWereBad(`What are you doing, printoconsole only takes bool!`);
    }
    printMsg = setting;
}
// ░██████╗███████╗████████╗██╗░░░██╗██████╗░
// ██╔════╝██╔════╝╚══██╔══╝██║░░░██║██╔══██╗
// ╚█████╗░█████╗░░░░░██║░░░██║░░░██║██████╔╝
// ░╚═══██╗██╔══╝░░░░░██║░░░██║░░░██║██╔═══╝░
// ██████╔╝███████╗░░░██║░░░╚██████╔╝██║░░░░░
// ╚═════╝░╚══════╝░░░╚═╝░░░░╚═════╝░╚═╝░░░░░
export function addService(name, jsonFilename, rateOpts = {}) {
    if (typeof name !== "string" || name === "") {
        itWereBad("name gotta be a string yo");
    }
    if (typeof jsonFilename !== "string" || jsonFilename === "") {
        itWereBad("JSON name gotta be a string yo");
    }
    jsonFilename = jsonFilename.replace(/\.json$/i, "");
    let cases;
    try {
        cases = JSON.parse(fs.readFileSync(`${cwd}\\src\\tests\\cases\\${jsonFilename}.json`, 'utf8'));
    }
    catch (e) {
        itWereBad(`Failed to load ${jsonFilename} - make sure it's in the root project directory/src/tests/cases/\n${e.message}`);
    }
    if (Object.keys(rateOpts).length === 0) {
        rateOpts = {
            enabled: false,
            rate: 0,
            nextWindow: new Date(0)
        };
    }
    else {
        if (typeof rateOpts.enabled !== 'boolean' || typeof rateOpts.rate !== 'number' || !(rateOpts.nextWindow instanceof Date)) {
            throw new Error('Rate opts gotta be enabled: bool, rate: numnber, nextWindow: date');
        }
    }
    mockQueue[name] = {
        get: [],
        put: [],
        patch: [],
        post: [],
        records: {
            lastBody: null,
            lastUrl: null
        },
        counts: {
            get: 0,
            put: 0,
            patch: 0,
            post: 0
        },
        cases: cases,
        rateLimiting: rateOpts
    };
}
export function addToQueue(service, method, newQ) {
    if (!mockQueue.hasOwnProperty(service)) {
        itWereBad(`Tried to add to service ${service} - gotta load it first dingus`);
    }
    if (typeof newQ.caseName !== "string" || newQ.caseName === "") {
        itWereBad("caseName gotta be a string yo");
    }
    if (newQ.hasOwnProperty("logicFunction")) {
        if (typeof (newQ.logicFunction) !== 'function' && typeof (newQ.logicFunction) !== null) {
            itWereBad("logicFunction gotta be non-existant, function, or null yo!");
        }
    }
    mockQueue[service][method].push(newQ);
}
export function resetQueue() {
    for (const service of Object.keys(mockQueue)) {
        mockQueue[service].get = [];
        mockQueue[service].post = [];
        mockQueue[service].put = [];
        mockQueue[service].patch = [];
        mockQueue[service].records.lastBody = null;
        mockQueue[service].records.lastUrl = null;
        mockQueue[service].counts.get = 0;
        mockQueue[service].counts.post = 0;
        mockQueue[service].counts.put = 0;
    }
}
function loadJSON(mockDeets, nextQueueItem, mockService) {
    if (!mockDeets.cases.hasOwnProperty(nextQueueItem.caseName)) {
        itWereBad(`Service ${mockService} doesnt have a case named ${nextQueueItem.caseName}`);
    }
    const response = mockDeets.cases[nextQueueItem.caseName];
    if (!response.hasOwnProperty('body') || !response.hasOwnProperty('status')) {
        throw new Error(`Call to mock ${mockService} ${nextQueueItem.caseName} was missing either body or status!`);
    }
    return response;
}
// ██████╗░░█████╗░  ███╗░░░███╗░█████╗░░█████╗░██╗░░██╗  ░█████╗░██████╗░██╗
// ██╔══██╗██╔══██╗  ████╗░████║██╔══██╗██╔══██╗██║░██╔╝  ██╔══██╗██╔══██╗██║
// ██║░░██║███████║  ██╔████╔██║██║░░██║██║░░╚═╝█████═╝░  ███████║██████╔╝██║
// ██║░░██║██╔══██║  ██║╚██╔╝██║██║░░██║██║░░██╗██╔═██╗░  ██╔══██║██╔═══╝░██║
// ██████╔╝██║░░██║  ██║░╚═╝░██║╚█████╔╝╚█████╔╝██║░╚██╗  ██║░░██║██║░░░░░██║
// ╚═════╝░╚═╝░░╚═╝  ╚═╝░░░░░╚═╝░╚════╝░░╚════╝░╚═╝░░╚═╝  ╚═╝░░╚═╝╚═╝░░░░░╚═╝
function mockAPI(opts) {
    knownErr = false;
    try {
        let bodyType = 'json';
        if (opts.url.match(/slack/i)) {
            let matched = false;
            if (typeof opts.body === 'string') {
                matched = !!(opts.body.match(/handle 418/i));
            }
            else {
                matched = !!(JSON.stringify(opts.body).match(/handle 418/i));
            }
            if (matched) {
                return { status: 200, body: '' };
            }
        }
        const intecreptedMethod = validateMethod(opts);
        // fetch vs http mocking may have an object or a string as the body :(
        if (opts.body) {
            if (typeof opts.body === 'string') {
                opts.body = JSON.parse(opts.body);
            }
            else if (opts.body instanceof URLSearchParams) {
                bodyType = 'urlparam';
                opts.body = opts.body.toString();
            }
        }
        if (printMsg) {
            console.log(`\n\nIntercepting ${opts.method} to ${opts.url}`);
            if (intecreptedMethod === "post" || intecreptedMethod === "put" || intecreptedMethod === "patch") {
                if (opts.body) {
                    if (bodyType === 'json') {
                        console.log(JSON.stringify(opts.body, null, 3));
                    }
                    else if (bodyType === 'urlparam') {
                        console.log(opts.body);
                    }
                }
            }
        }
        let mockService = findMockedService(mockQueue, opts);
        if (!mockService) {
            return null;
        }
        if (checkRateLimits(mockService)) {
            return { status: 429, body: "Ooops" };
        }
        let mockDeets = mockQueue[mockService];
        mockDeets.counts[intecreptedMethod]++;
        let nextQueueItem = mockDeets[intecreptedMethod].shift();
        if (!nextQueueItem) {
            itWereBad(`Queue for ${mockService}/${intecreptedMethod} was empty`);
        }
        validateRequiredFields(nextQueueItem, opts, bodyType);
        runExtraLogic(nextQueueItem, opts);
        return loadJSON(mockDeets, nextQueueItem, mockService);
    }
    catch (e) {
        if (!knownErr) {
            console.log(`Unexpected error: ${e.message}\n${e.stack}`);
        }
        return { status: 418, body: e.message, trace: e.stack };
    }
}
function checkRateLimits(mockServiceName) {
    if (mockQueue[mockServiceName].rateLimiting.enabled) {
        if (mockQueue[mockServiceName].rateLimiting.nextWindow > (new Date())) {
            return true;
        }
        mockQueue[mockServiceName].rateLimiting.nextWindow = new Date(Number(new Date()) + (1 / mockQueue[mockServiceName].rateLimiting.rate));
    }
    return false;
}
function validateMethod(opts) {
    const intecreptedMethod = opts.method.toLowerCase();
    if (!["get", "post", "put", "patch"].includes(intecreptedMethod)) {
        itWereBad(`method ${intecreptedMethod} unsupported`);
    }
    return intecreptedMethod;
}
function findMockedService(mockQueue, opts) {
    let mockService = "";
    for (const [serviceName, serviceDeets] of Object.entries(mockQueue)) {
        let urlRegex = new RegExp(serviceName, "i");
        if (opts.url.match(urlRegex)) {
            mockService = serviceName;
            break;
        }
    }
    if (mockService === "") {
        return null;
    }
    if (!mockQueue[mockService]) {
        itWereBad(`Deets werent defiend for ${mockService}`);
    }
    return mockService;
}
function validateRequiredFields(nextQueueItem, opts, bodyType) {
    var _a, _b, _c, _d;
    if (nextQueueItem.hasOwnProperty("requirements")) {
        if ((_a = nextQueueItem.requirements) === null || _a === void 0 ? void 0 : _a.url) {
            if (nextQueueItem.requirements.url instanceof RegExp) {
                if (!opts.url.match(nextQueueItem.requirements.url)) {
                    const stringComp = compareStrings(String(nextQueueItem.requirements.url).replace(/\\(.)/g, "$1").replace(/(?:^\/|\/$)/g, ''), opts.url || "");
                    itWereBad(`URL match failure - expected:\n${stringComp.str1}\n\nbut got: \n${stringComp.str2}`);
                }
            }
            else {
                if (opts.url !== nextQueueItem.requirements.url) {
                    const stringComp = compareStrings(nextQueueItem.requirements.url, opts.url || "");
                    itWereBad(`URL match failure - expected:\n${stringComp.str1}\n\nbut got:\n${stringComp.str2}`);
                }
            }
        }
        if ((_b = nextQueueItem.requirements) === null || _b === void 0 ? void 0 : _b.body) {
            let requestBody = JSON.stringify(opts.body);
            let requiredBody = nextQueueItem.requirements.body;
            if (bodyType === 'json') {
                requestBody = JSON.stringify(opts.body);
                requiredBody = nextQueueItem.requirements.body;
            }
            else if (bodyType === 'urlparam') {
                requestBody = opts.body;
                requiredBody = nextQueueItem.requirements.body;
            }
            //        console.log(`${requestBody} \nvs\n${requiredBody}\ngives: ${requestBody.match(requiredBody)}`)
            if (requestBody !== requiredBody) {
                const stringComp = compareStrings(nextQueueItem.requirements.body, JSON.stringify(opts.body) || "");
                if (!stringComp.wildcardMatch) {
                    itWereBad(`Body match failure - expected:\n${stringComp.str1}\n\nbut got:\n${stringComp.str2}`);
                }
            }
        }
        if ((_c = nextQueueItem.requirements) === null || _c === void 0 ? void 0 : _c.headers) {
            if (!opts.headers || Object.keys(opts.headers).length === 0) {
                itWereBad("Intercepted thing had no headers");
            }
            for (const [reqHeaderName, reqHeaderVal] of Object.entries((_d = nextQueueItem.requirements) === null || _d === void 0 ? void 0 : _d.headers)) {
                //@ts-ignore
                if (!opts.headers.hasOwnProperty(reqHeaderName)) {
                    itWereBad(`Intercepted thing was missing required header ${printColor(reqHeaderName, "blue")}`);
                }
                //@ts-ignore
                if (opts.headers[reqHeaderName] !== reqHeaderVal) {
                    //@ts-ignore
                    const stringComp = compareStrings(reqHeaderVal, JSON.stringify(opts.headers[reqHeaderName]) || "");
                    itWereBad(`Intercepted thing header ${printColor(reqHeaderName, "blue")} expected:\n"${stringComp.str1}"\n\nbut got:\n${stringComp.str2}`);
                }
            }
        }
    }
}
function runExtraLogic(nextQueueItem, opts) {
    if (nextQueueItem.hasOwnProperty("logicFunction")) {
        let result = false;
        try {
            result = nextQueueItem.logicFunction(opts);
        }
        catch (e) {
            itWereBad(`Something on the custom function exploded - it's gotta take an arg of RequestInfo and return bool - a false throws errors\n${e}`);
        }
        if (!result) {
            itWereBad(`A function condition failed`);
        }
    }
}
function itWereBad(msg) {
    knownErr = true;
    if (printMsg) {
        console.log(`<MOCK API ERR>\n${msg}`);
    }
    throw new Error(msg);
}
export function compareStrings(requirementStr, strToCheck) {
    let requiredArr = requirementStr.match(/./g) || [];
    let checkedArr = strToCheck.match(/./g) || [];
    let longestArr = [requiredArr.length, checkedArr.length].sort(function (a, b) { return a - b; })[1];
    let printSt1 = "";
    let printSt2 = "";
    let failedNonWildcard = false;
    for (let i = 0; i < longestArr; i++) {
        if (requiredArr[i] === checkedArr[i]) {
            printSt1 = `${printSt1}${requiredArr[i]}`;
            printSt2 = `${printSt2}${requiredArr[i]}`;
        }
        else {
            if (requiredArr[i] === undefined) {
                failedNonWildcard = true;
                printSt1 = `${printSt1} `;
                printSt2 = `${printSt2}${printColor(checkedArr[i], "red")}`;
            }
            else if (checkedArr[i] === undefined) {
                failedNonWildcard = true;
                printSt1 = `${printSt1}${printColor(requiredArr[i], "red")}`;
                printSt2 = `${printSt2} `;
            }
            else {
                if (requiredArr[i] === "*") {
                    printSt1 = `${printSt1}${printColor(requiredArr[i], "blue")}`;
                    printSt2 = `${printSt2}${printColor(checkedArr[i], "blue")}`;
                }
                else {
                    failedNonWildcard = true;
                    printSt1 = `${printSt1}${printColor(requiredArr[i], "red")}`;
                    printSt2 = `${printSt2}${printColor(checkedArr[i], "red")}`;
                }
            }
        }
    }
    return { str1: printSt1, str2: printSt2, wildcardMatch: !failedNonWildcard };
}
function printColor(input, color) {
    switch (color) {
        case "red":
            return `\x1b[91m${input}\x1b[0m`;
            break;
        case "blue":
            return `\x1b[94m${input}\x1b[0m`;
            break;
        default:
            throw `unknown color ${color}`;
    }
}
