var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as apiMocker from "../makeapimockery.js";
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Who tests the tester', () => __awaiter(void 0, void 0, void 0, function* () {
    context('It All Works', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Starts, detects missing service, stops', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.start();
            let testResponse = yield fetch("http://www.vocm.com");
            expect(testResponse.status).to.equal(200);
            //expect(await testResponse.text()).to.equal(`Couldn't find service`)
            apiMocker.stop();
            testResponse = yield fetch("http://www.vocm.com");
            expect(testResponse.status).to.equal(200);
        }));
        it('Validates adding a new service', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            expect(() => { apiMocker.addService(15); }).to.throw("name gotta be a string yo");
            //@ts-ignore
            expect(() => { apiMocker.addService(""); }).to.throw("name gotta be a string yo");
            //@ts-ignore
            expect(() => { apiMocker.addService("Thing", null); }).to.throw("JSON name gotta be a string yo");
            expect(() => { apiMocker.addService("Thing", ""); }).to.throw("JSON name gotta be a string yo");
            expect(() => { apiMocker.addService("testService", "qqqqq.json"); }).to.throw("Failed to load qqqqq - make sure it's in the root project directory/src/tests/cases/");
            apiMocker.addService("testAPI", "testJSON.json");
        }));
        it('Validates a bad queue request', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.start();
            let badQ = {
                caseName: "ffff"
            };
            apiMocker.addToQueue("testAPI", "get", badQ);
            let testReq = yield fetch("http://www.testapi.org/thing");
            expect(testReq.status).to.equal(418);
            expect(yield testReq.text()).to.equal("Service testAPI doesnt have a case named ffff");
            testReq = yield fetch("http://www.testapi.org/thing", { method: "POST" });
            expect(testReq.status).to.equal(418);
            expect(yield testReq.text()).to.equal("Queue for testAPI/post was empty");
        }));
        it('Works - basic get', () => __awaiter(void 0, void 0, void 0, function* () {
            let goodQ = {
                caseName: "testMatch"
            };
            apiMocker.addToQueue("testAPI", "get", goodQ);
            expect(apiMocker.getCounts("testAPI").get).to.equal(1);
            let testReq = yield fetch("http://www.testapi.org/thing");
            expect(testReq.status).to.equal(403);
            expect(yield testReq.text()).to.equal("Hello this worked well!");
            expect(apiMocker.getCounts("testAPI").get).to.equal(2);
            apiMocker.resetQueue();
            expect(apiMocker.getCounts("testAPI").get).to.equal(0);
        }));
        it('Does URL validation', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.addToQueue("testAPI", "get", { caseName: "testMatch", requirements: { url: "http://www.testapi.org/lolwtf" } });
            apiMocker.addToQueue("testAPI", "get", { caseName: "testMatch", requirements: { url: "http://www.testapi.org/thing" } });
            apiMocker.addToQueue("testAPI", "get", { caseName: "testMatch", requirements: { url: new RegExp("^.+?tEsTapi.+?/thing$") } });
            apiMocker.addToQueue("testAPI", "get", { caseName: "testMatch", requirements: { url: new RegExp("^.+?tEsTapi.+?/thing$", "i") } });
            let testReq = yield fetch("http://www.testapi.org/thing");
            expect(testReq.status).to.equal(418);
            expect(yield testReq.text()).to.include("URL match failure - expected");
            testReq = yield fetch("http://www.testapi.org/thing");
            expect(testReq.status).to.equal(403);
            testReq = yield fetch("http://www.testapi.org/thing");
            expect(testReq.status).to.equal(418);
            testReq = yield fetch("http://www.testapi.org/thing");
            expect(testReq.status).to.equal(403);
        }));
        it('Does Body validation', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", requirements: { body: JSON.stringify({ key: "badValue" }) } });
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", requirements: { body: JSON.stringify({ key: "goodValue" }) } });
            let test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST", body: JSON.stringify({ key: "goodValue" }) });
            expect(test.status).to.equal(418);
            expect(yield test.text()).to.include("Body match failure - expected");
            test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST", body: JSON.stringify({ key: "goodValue" }) });
            expect(test.status).to.equal(201);
            expect(apiMocker.getCounts("testAPI").post).to.equal(2);
        }));
        it('Does Body validation with wildcards', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", requirements: { body: JSON.stringify({ key: "badValue" }) } });
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", requirements: { body: JSON.stringify({ key: "****Value" }) } });
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", requirements: { body: JSON.stringify({ key: "****Value" }) } });
            let test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST", body: JSON.stringify({ key: "goodValue" }) });
            expect(test.status).to.equal(418);
            expect(yield test.text()).to.include("Body match failure - expected");
            test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST", body: JSON.stringify({ key: "someOtherValue" }) });
            expect(test.status).to.equal(418);
            expect(yield test.text()).to.include("Body match failure - expected");
            test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST", body: JSON.stringify({ key: "q{q?Value" }) });
            expect(test.status).to.equal(201);
            expect(apiMocker.getCounts("testAPI").post).to.equal(3);
        }));
        it('Does Header validation', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            apiMocker.addToQueue("testAPI", "put", { caseName: "yert", requirements: { headers: { "header1": "val1", "header2": "val2" } } });
            let test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "PUT", headers: {} });
            expect(yield test.text()).to.include("Intercepted thing had no headers");
            apiMocker.addToQueue("testAPI", "put", { caseName: "yert", requirements: { headers: { "qqqq": "asdsada", "header2": "val2" } } });
            test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "PUT", headers: { "header1": "val2", "header2": "val2" } });
            expect(yield test.text()).to.include("Intercepted thing was missing required");
            apiMocker.addToQueue("testAPI", "put", { caseName: "yert", requirements: { headers: { "header1": "val2", "header2": "val2" } } });
            test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "PUT", headers: { "header1": "val1", "header2": "val2" } });
            expect(yield test.text()).to.include('Intercepted thing header');
        }));
        it('Uses the function feature', () => __awaiter(void 0, void 0, void 0, function* () {
            let testFunctionFails = (opts) => {
                if (opts.url !== "sadadsadadad") {
                    return false;
                }
                return true;
            };
            let testFunctionOK = (opts) => {
                return true;
            };
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", logicFunction: testFunctionFails });
            apiMocker.addToQueue("testAPI", "post", { caseName: "postWorks", logicFunction: testFunctionOK });
            let test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST" });
            expect(yield test.text()).to.include("A function condition failed");
            test = yield fetch("http://www.testapi.org/api/v99/hello", { method: "POST" });
            expect(test.status).to.equal(201);
        }));
        it('Gets string diffs', () => __awaiter(void 0, void 0, void 0, function* () {
            const json1 = {
                name: "John",
                age: 30,
                address: {
                    street: "123 Main St",
                    city: "Anytown"
                },
                andLast: "value"
            };
            const json2 = {
                name: "John",
                age: 31,
                address: {
                    street: "123 Main St",
                    city: "Newtown",
                    extraThing: "yert"
                },
                andLast: "value"
            };
            const differences = apiMocker.compareStrings(JSON.stringify(json1), JSON.stringify(json2));
            expect(JSON.stringify(differences.str1)).to.equal('"{\\"name\\":\\"John\\",\\"age\\":3\\u001b[91m0\\u001b[0m,\\"address\\":{\\"street\\":\\"123 Main St\\",\\"city\\":\\"\\u001b[91mA\\u001b[0m\\u001b[91mn\\u001b[0m\\u001b[91my\\u001b[0mtown\\"\\u001b[91m}\\u001b[0m\\u001b[91m,\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91mn\\u001b[0m\\u001b[91md\\u001b[0m\\u001b[91mL\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91ms\\u001b[0m\\u001b[91mt\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91m:\\u001b[0m\\"\\u001b[91mv\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91ml\\u001b[0m\\u001b[91mu\\u001b[0m\\u001b[91me\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91m}\\u001b[0m                    "');
            expect(JSON.stringify(differences.str2)).to.equal('"{\\"name\\":\\"John\\",\\"age\\":3\\u001b[91m1\\u001b[0m,\\"address\\":{\\"street\\":\\"123 Main St\\",\\"city\\":\\"\\u001b[91mN\\u001b[0m\\u001b[91me\\u001b[0m\\u001b[91mw\\u001b[0mtown\\"\\u001b[91m,\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91me\\u001b[0m\\u001b[91mx\\u001b[0m\\u001b[91mt\\u001b[0m\\u001b[91mr\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91mT\\u001b[0m\\u001b[91mh\\u001b[0m\\u001b[91mi\\u001b[0m\\u001b[91mn\\u001b[0m\\u001b[91mg\\u001b[0m\\"\\u001b[91m:\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91my\\u001b[0m\\u001b[91me\\u001b[0m\\u001b[91mr\\u001b[0m\\u001b[91mt\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91m}\\u001b[0m\\u001b[91m,\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91mn\\u001b[0m\\u001b[91md\\u001b[0m\\u001b[91mL\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91ms\\u001b[0m\\u001b[91mt\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91m:\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91mv\\u001b[0m\\u001b[91ma\\u001b[0m\\u001b[91ml\\u001b[0m\\u001b[91mu\\u001b[0m\\u001b[91me\\u001b[0m\\u001b[91m\\"\\u001b[0m\\u001b[91m}\\u001b[0m"');
        }));
        it('Returns a 200 when sending slack 418 warnings', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            apiMocker.addService("slack", "testJSON.json");
            apiMocker.addToQueue("slack", "put", { caseName: "yert", requirements: { headers: { "header1": "val1", "header2": "val2" } } });
            let test = yield fetch("https://hooks.slack.com/blahgblahlbagasd/", { method: "POST", body: '{"someMessage": "418 bottles of beer on the wall!"}' });
            expect(test.status).to.equal(418);
            test = yield fetch("https://hooks.slack.com/blahgblahlbagasd/", { method: "POST", body: '{"someMessage": "handle 418 error!"}' });
            expect(test.status).to.equal(200);
        }));
        it('Returns custom headers if set in the JSON', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            apiMocker.addService("yert", "testJSON.json");
            apiMocker.addToQueue("yert", "get", { caseName: "customHeaders" });
            let test = yield fetch("https://yert.yert/yert", { method: "GET" });
            expect(test.headers.get('heder1')).to.equal('val4');
            expect(test.headers.get('header2')).to.equal('spelled right');
            expect(test.headers.get('blah-blah')).to.equal('123456');
            expect(test.status).to.equal(200);
            expect(yield test.text()).to.equal('There gonna be headers!');
        }));
    }));
    context('Printing outstanding queue', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Prints the right string for an empty queue', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            expect(apiMocker.printQueue()).to.equal("No outstanding Q items");
        }));
        it('Prints the right string for a non-empty queue', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.resetQueue();
            expect(apiMocker.printQueue()).to.equal("No outstanding Q items");
            apiMocker.addToQueue("slack", "put", { caseName: "thing two" });
            apiMocker.addToQueue("slack", "get", { caseName: "thing one" });
            apiMocker.addToQueue("slack", "put", { caseName: "thing three" });
            const expectedStr = "Service: slack\n  get -> thing one\n  put -> thing two\n  put -> thing three";
            expect(apiMocker.printQueue()).to.equal(expectedStr);
        }));
        it('Prints the right string for no services', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.stop();
            expect(apiMocker.printQueue()).to.equal("No Services");
        }));
    }));
    context('Testing the rate limiting feature', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Works', () => __awaiter(void 0, void 0, void 0, function* () {
            apiMocker.printToConsole(true);
            apiMocker.start();
            apiMocker.addService("ffff", "testJSON.json", { enabled: true, rate: 0.5, nextWindow: new Date() });
            apiMocker.addToQueue("ffff", "get", { caseName: "customHeaders" });
            apiMocker.addToQueue("ffff", "get", { caseName: "customHeaders" });
            yield expect(fetch("https://ffff.com/ffff", { method: "GET" })).not.to.be.rejected;
            let reject = yield expect(fetch("https://ffff.com/ffff", { method: "GET" })).not.to.be.rejected;
            expect(reject.status).to.equal(429);
        }));
    }));
}));
